@@ -1,10 +1,10 @@
 ;;; erc-backend.el --- Backend network communication for ERC  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2004-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2004-2020 Free Software Foundation, Inc.
 
 ;; Filename: erc-backend.el
 ;; Author: Lawrence Mitchell <wence@gmx.li>
-;; Maintainer: emacs-devel@gnu.org
+;; Maintainer: Amin Bandali <bandali@gnu.org>
 ;; Created: 2004-05-7
 ;; Keywords: IRC chat client internet
 
@@ -261,7 +261,7 @@ but has not been processed yet.")
   "Non-nil when we're currently processing a message.
 
 When ERC receives a private message, it sets up a new buffer for
-this query.  These in turn, though, do start flyspell. This
+this query.  These in turn, though, do start flyspell.  This
 involves starting an external process, in which case Emacs will
 wait - and when it waits, it does accept other stuff from, say,
 network exceptions.  So, if someone sends you two messages
@@ -320,7 +320,7 @@ If a key is pressed while ERC is waiting, it will stop waiting."
   "The maximum length of a single message.
 If a message exceeds this size, it is broken into multiple ones.
 
-IRC allows for lines up to 512 bytes. Two of them are CR LF.
+IRC allows for lines up to 512 bytes.  Two of them are CR LF.
 And a typical message looks like this:
 
   :nicky!uhuser@host212223.dialin.fnordisp.net PRIVMSG #lazybastards :Hello!
@@ -347,8 +347,8 @@ This will only be consulted if the coding system in
 This is either a coding system, a cons, a function, or nil.
 
 If a cons, the encoding system for outgoing text is in the car
-and the decoding system for incoming text is in the cdr. The most
-interesting use for this is to put `undecided' in the cdr. This
+and the decoding system for incoming text is in the cdr.  The most
+interesting use for this is to put `undecided' in the cdr.  This
 means that `erc-coding-system-precedence' will be consulted, and the
 first match there will be used.
 
@@ -466,24 +466,29 @@ If this is set to nil, never try to reconnect."
 The length is specified in `erc-split-line-length'.
 
 Currently this is called by `erc-send-input'."
-  (if (< (length longline)
-         erc-split-line-length)
-      (list longline)
+  (let* ((coding (erc-coding-system-for-target nil))
+         (charset (if (consp coding) (car coding) coding)))
     (with-temp-buffer
       (insert longline)
+      ;; The line lengths are in octets, not characters (because these
+      ;; are server protocol limits), so we have to first make the
+      ;; text into bytes, then fold the bytes on "word" boundaries,
+      ;; and then make the bytes into text again.
+      (encode-coding-region (point-min) (point-max) charset)
       (let ((fill-column erc-split-line-length))
         (fill-region (point-min) (point-max)
                      nil t))
+      (decode-coding-region (point-min) (point-max) charset)
       (split-string (buffer-string) "\n"))))
 
 (defun erc-forward-word ()
-  "Moves forward one word, ignoring any subword settings.  If no
-subword-mode is active, then this is (forward-word)."
+  "Move forward one word, ignoring any subword settings.
+If no subword-mode is active, then this is (forward-word)."
   (skip-syntax-forward "^w")
   (> (skip-syntax-forward "w") 0))
 
 (defun erc-word-at-arg-p (pos)
-  "Reports whether the char after a given POS has word syntax.
+  "Report whether the char after a given POS has word syntax.
 If POS is out of range, the value is nil."
   (let ((c (char-after pos)))
     (if c
@@ -491,9 +496,9 @@ If POS is out of range, the value is nil."
       nil)))
 
 (defun erc-bounds-of-word-at-point ()
-  "Returns the bounds of a word at point, or nil if we're not at
-a word.  If no subword-mode is active, then this
-is (bounds-of-thing-at-point 'word)."
+  "Return the bounds of word at point, or nil if we're not at a word.
+If no subword-mode is active, then this is
+\(bounds-of-thing-at-point 'word)."
   (if (or (erc-word-at-arg-p (point))
           (erc-word-at-arg-p (1- (point))))
       (save-excursion
@@ -592,7 +597,7 @@ We will store server variables in the buffer given by BUFFER."
       (erc-login)) ))
 
 (defun erc-server-reconnect ()
-"Reestablish the current IRC connection.
+  "Reestablish the current IRC connection.
 Make sure you are in an ERC buffer when running this."
   (let ((buffer (erc-server-buffer)))
     (unless (buffer-live-p buffer)
@@ -644,22 +649,24 @@ Make sure you are in an ERC buffer when running this."
             (erc-log-irc-protocol line nil)
             (erc-parse-server-response process line)))))))
 
-(defsubst erc-server-reconnect-p (event)
+(define-inline erc-server-reconnect-p (event)
   "Return non-nil if ERC should attempt to reconnect automatically.
 EVENT is the message received from the closed connection process."
-  (or erc-server-reconnecting
-      (and erc-server-auto-reconnect
-           (not erc-server-banned)
-           ;; make sure we don't infinitely try to reconnect, unless the
-           ;; user wants that
-           (or (eq erc-server-reconnect-attempts t)
-               (and (integerp erc-server-reconnect-attempts)
-                    (< erc-server-reconnect-count
-                       erc-server-reconnect-attempts)))
-           (or erc-server-timed-out
-               (not (string-match "^deleted" event)))
-           ;; open-network-stream-nowait error for connection refused
-           (if (string-match "^failed with code 111" event) 'nonblocking t))))
+  (inline-letevals (event)
+    (inline-quote
+     (or erc-server-reconnecting
+         (and erc-server-auto-reconnect
+              (not erc-server-banned)
+              ;; make sure we don't infinitely try to reconnect, unless the
+              ;; user wants that
+              (or (eq erc-server-reconnect-attempts t)
+                  (and (integerp erc-server-reconnect-attempts)
+                       (< erc-server-reconnect-count
+                          erc-server-reconnect-attempts)))
+              (or erc-server-timed-out
+                  (not (string-match "^deleted" ,event)))
+              ;; open-network-stream-nowait error for connection refused
+              (if (string-match "^failed with code 111" ,event) 'nonblocking t))))))
 
 (defun erc-process-sentinel-2 (event buffer)
   "Called when `erc-process-sentinel-1' has detected an unexpected disconnect."
@@ -759,8 +766,8 @@ This is determined via `erc-encoding-coding-alist' or
 
 (defun erc-decode-string-from-target (str target)
   "Decode STR as appropriate for TARGET.
-This is indicated by `erc-encoding-coding-alist', defaulting to the value of
-`erc-server-coding-system'."
+This is indicated by `erc-encoding-coding-alist', defaulting to the
+value of `erc-server-coding-system'."
   (unless (stringp str)
     (setq str ""))
   (let ((coding (erc-coding-system-for-target target)))
@@ -789,7 +796,7 @@ Use DISPLAY-FN to show the results."
 (defun erc-server-send (string &optional forcep target)
   "Send STRING to the current server.
 If FORCEP is non-nil, no flood protection is done - the string is
-sent directly. This might cause the messages to arrive in a wrong
+sent directly.  This might cause the messages to arrive in a wrong
 order.
 
 If TARGET is specified, look up encoding information for that
@@ -838,10 +845,9 @@ Additionally, detect whether the IRC process has hung."
              erc-server-last-received-time))
       (with-current-buffer buf
         (if (and erc-server-send-ping-timeout
-                 (>
-                  (erc-time-diff (erc-current-time)
-                                 erc-server-last-received-time)
-                  erc-server-send-ping-timeout))
+                 (time-less-p
+                  erc-server-send-ping-timeout
+                  (time-since erc-server-last-received-time)))
             (progn
               ;; if the process is hung, kill it
               (setq erc-server-timed-out t)
@@ -859,16 +865,15 @@ Additionally, detect whether the IRC process has hung."
 See `erc-server-flood-margin' for an explanation of the flood
 protection algorithm."
   (with-current-buffer buffer
-    (let ((now (erc-current-time)))
+    (let ((now (current-time)))
       (when erc-server-flood-timer
         (erc-cancel-timer erc-server-flood-timer)
         (setq erc-server-flood-timer nil))
-      (when (< erc-server-flood-last-message
-               now)
-        (setq erc-server-flood-last-message now))
+      (when (time-less-p erc-server-flood-last-message now)
+        (setq erc-server-flood-last-message (erc-emacs-time-to-erc-time now)))
       (while (and erc-server-flood-queue
-                  (< erc-server-flood-last-message
-                     (+ now erc-server-flood-margin)))
+                  (time-less-p erc-server-flood-last-message
+                               (time-add now erc-server-flood-margin)))
         (let ((msg (caar erc-server-flood-queue))
               (encoding (cdar erc-server-flood-queue)))
           (setq erc-server-flood-queue (cdr erc-server-flood-queue)
@@ -899,7 +904,7 @@ protection algorithm."
   "Send LINE to the server as a privmsg or a notice.
 MESSAGE-COMMAND should be either \"PRIVMSG\" or \"NOTICE\".
 If the target is \",\", the last person you've got a message from will
-be used. If the target is \".\", the last person you've sent a message
+be used.  If the target is \".\", the last person you've sent a message
 to will be used."
   (cond
    ((string-match "^\\s-*\\(\\S-+\\) ?\\(.*\\)" line)
@@ -1064,8 +1069,8 @@ Hands off to helper functions via `erc-call-hooks'."
               erc-server-prevent-duplicates)
       (let ((m (erc-response.unparsed parsed-response)))
         ;; duplicate suppression
-        (if (< (or (gethash m erc-server-duplicates) 0)
-               (- (erc-current-time) erc-server-duplicate-timeout))
+        (if (time-less-p (or (gethash m erc-server-duplicates) 0)
+                         (time-since erc-server-duplicate-timeout))
             (erc-call-hooks process parsed-response))
         (puthash m (erc-current-time) erc-server-duplicates))
     ;; Hand off to the relevant handler.
@@ -1114,7 +1119,8 @@ NAME is the response name as sent by the server (see the IRC RFC for
 meanings).
 
 This creates:
- - a hook variable `erc-server-NAME-functions' initialized to `erc-server-NAME'.
+ - a hook variable `erc-server-NAME-functions' initialized to
+   `erc-server-NAME'.
  - a function `erc-server-NAME' with body FN-BODY.
 
 If ALIASES is non-nil, each alias in ALIASES is `defalias'ed to
@@ -1281,7 +1287,7 @@ add things to `%s' instead."
     (pcase-let ((`(,nick ,login ,host)
                  (erc-parse-user (erc-response.sender parsed))))
       ;; strip the stupid combined JOIN facility (IRC 2.9)
-      (if (string-match "^\\(.*\\)?\^g.*$" chnl)
+      (if (string-match "^\\(.*\\)\^g.*$" chnl)
           (setq chnl (match-string 1 chnl)))
       (save-excursion
         (let* ((str (cond
@@ -1441,7 +1447,7 @@ add things to `%s' instead."
   "Handle pong messages." nil
   (let ((time (string-to-number (erc-response.contents parsed))))
     (when (> time 0)
-      (setq erc-server-lag (erc-time-diff time (erc-current-time)))
+      (setq erc-server-lag (erc-time-diff time nil))
       (when erc-verbose-server-ping
         (erc-display-message
          parsed 'notice proc 'PONG
@@ -1724,7 +1730,7 @@ See `erc-display-server-message'." nil
                (cdr (erc-response.command-args parsed))))
     (setq time (when on-since
                  (format-time-string erc-server-timestamp-format
-                                     (erc-string-to-emacs-time on-since))))
+                                     (string-to-number on-since))))
     (erc-update-user-nick nick nick nil nil nil
                           (and time (format "on since %s" time)))
     (if time
@@ -1796,7 +1802,7 @@ See `erc-display-server-message'." nil
 (define-erc-response-handler (329)
   "Channel creation date." nil
   (let ((channel (cadr (erc-response.command-args parsed)))
-        (time (erc-string-to-emacs-time
+        (time (string-to-number
                (nth 2 (erc-response.command-args parsed)))))
     (erc-display-message
      parsed 'notice (erc-get-buffer channel proc)
@@ -1838,7 +1844,7 @@ See `erc-display-server-message'." nil
   (pcase-let ((`(,channel ,nick ,time)
                (cdr (erc-response.command-args parsed))))
     (setq time (format-time-string erc-server-timestamp-format
-                                   (erc-string-to-emacs-time time)))
+                                   (string-to-number time)))
     (erc-update-channel-topic channel
                               (format "\C-o (%s, %s)" nick time)
                               'append)
@@ -2071,6 +2077,3 @@ See `erc-display-error-notice'." nil
 (provide 'erc-backend)
 
 ;;; erc-backend.el ends here
-;; Local Variables:
-;; indent-tabs-mode: nil
-;; End:
