@@ -1,6 +1,6 @@
 ;;; greek.el --- Quail package for inputting Greek -*-coding: utf-8-*-
 
-;; Copyright (C) 2001-2015 Free Software Foundation, Inc.
+;; Copyright (C) 2001-2020 Free Software Foundation, Inc.
 ;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 ;;   2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -21,7 +21,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -178,8 +178,8 @@ koppa		&		%
 mark		key
 ------------------------
 ypogegrammeni	J
-psili		'  or  v
-dasia		`  or  V
+psili		\\='  or  v
+dasia		\\=`  or  V
 oxia		/
 varia		?
 perispomeni	\\  or  ^
@@ -461,9 +461,9 @@ mark		key
 ypogegrammeni	|
 psili		>
 dasia		<
-oxia		'
-koronis         ''
-varia		`
+oxia		\\='
+koronis		\\='\\='
+varia		\\=`
 perispomeni	~
 dialytika	\"
 ano teleia	;
